/* =================================================================================
 *  alberi.c
 * =================================================================================
 * Contiene l'implementazione delle seguenti funzioni e procedure su alberi binari:
 * + costruzione di un albero binario inserito da input
 * + visita / stampa di un albero
 * + calcolo del numero delle foglie dell'albero (versione funzionale)
 * + calcolo del numero di nodi (versione funzionale e procedurale)
 * + calcolo dell'altezza (versione funzionale e procedurale)
 * + calcolo del livello della foglia piu' a sinistra (versione funzionale e procedurale)
 * + stampa in output indentata di un albero binario
 * + verifica se l'albero e' vuoto
 *
 *  Authors:     Emanuela Fachini (fachini@di.uniroma1.it)
 *               Irene Finocchi   (finocchi@di.uniroma1.it)
 *               Chiara Petrioli  (petrioli@di.uniroma1.it)
 *  Last change: 2 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#include "alberi.h"

/* Postc: restituisce il numero dei nodi di tPtr */
int numNodi(TreePtr tPtr) {
    if (!tPtr ) return 0;
    return  numNodi(tPtr->lPtr)  + numNodi(tPtr->rPtr) + 1;
}

/* Osservazione: n deve essere inizializzato a 0 */
/* Postc: n contiene il numero dei nodi di tPtr  */
void numNodi2(TreePtr tPtr, int *n) {
    if (!tPtr )  return;
    *(n) = *(n)+1;
    numNodi2(tPtr->lPtr,n); 
    numNodi2(tPtr->rPtr,n);
}


int max (int x, int y) {
    if (x<y) return y; 
    else return x;
}  

/* Postc: restituisce l'altezza di tPtr */
int alt(TreePtr tPtr) {
    if (!tPtr ) return -1;
    return  max(alt(tPtr->lPtr),alt(tPtr->rPtr)) +1;
}
 
/* Osservazione: h deve essere inizializzato a -1 */
/* Postc: h contiene l'altezza di tPtr */
void alt2(TreePtr tPtr, int* h) {
    int h1;
    if (!tPtr ) return;
    if (tPtr->lPtr) alt2(tPtr->lPtr,h);
    h1 = *h;*h=-1;
    if (tPtr->rPtr) alt2(tPtr->rPtr,h); 
    *h = max(h1,(*h))+1;
}


/* Postc: restituisce il livello della foglia pi a sinistra  di tPtr */
int livFoSin(TreePtr tPtr) {
    if (!tPtr) return -1;
    if (tPtr->lPtr==NULL && tPtr->rPtr==NULL) return 0;
    return 1+livFoSin(tPtr->lPtr ? tPtr->lPtr : tPtr->rPtr);
} 

/* Osservazione: liv deve essere inizializzato a -1 */
/*  postc: liv contiene il livello della foglia pi a sinistra  di tPtr */
void livFoSin2(TreePtr tPtr, int* liv) {
    if (!tPtr) return;
    (*liv)++;    
    if (tPtr->lPtr==NULL && tPtr->rPtr==NULL) return;
    livFoSin2(tPtr->lPtr? tPtr->lPtr : tPtr->rPtr,liv);
} 


/* Pre: l'albero che si costruisce e' NON vuoto*/
/* Post: costruisce in preordine un albero binario*/
TreePtr creaalbero() {
	TreePtr T;
	int i,j;

	scanf("%d %d",&i,&j);
	T = malloc(sizeof(TREENODE));

	if (T==NULL) printf("Impossibile allocare memoria per l'albero\n"); 
	else {
		T->elem=i;
		switch(j) {
			case 0:  T->rPtr=NULL;
				     T->lPtr=NULL;
				     break;
			case -1: T->lPtr=creaalbero();
				     T->rPtr=NULL;
				     break;
			case 1:  T->lPtr=NULL;
				     T->rPtr=creaalbero();
				     break;
			case 2:  T->lPtr=creaalbero();
				     T->rPtr=creaalbero();
				     break;
		}
	}
	return(T);
}


/* Post: restituisce 1 se l'albero e' vuoto, 0 altrimenti*/
int alberovuoto(TreePtr T) {
	return(T==NULL);
}

/* Post: restituisce il numero di foglie dell'albero binario, 0 se l'albero e' vuoto */
int numfoglie(TreePtr T) {
	if (T==NULL) return 0;
	else {
		if ((T->lPtr ==NULL) && (T->rPtr ==NULL)) return 1;
		else return (numfoglie(T->lPtr)+numfoglie(T->rPtr));
	}
}

/* Post: stampa in inorder i valori contenuti nei nodi dell'albero*/
void stampaAlberoInorder(TreePtr T) {
	if (T!=NULL) {
		stampaAlberoInorder(T->lPtr);
		printf("%d ",T->elem);
		stampaAlberoInorder(T->rPtr);
	}
	return;
}

/* OSSERVAZIONE: nel main chiamata con h a zero */
/* Post: stampa i valori dell'albero indentati  */
void stampaIndentata (TreePtr T, int h) {
	int i;
	if (T!=NULL) {
		stampaIndentata(T->rPtr,h+1);
		for (i=1;i<=h;i++)
			printf("\t");
		printf("%d \n", T->elem);
		stampaIndentata(T->lPtr,h+1);
	}
	
}


